@@ -1,20 +1,20 @@
-use core::{cell::Cell, fmt, marker::PhantomData};
+use core::{cell::Cell, fmt, marker::PhantomData, mem::MaybeUninit, ops::Deref};
 
 use crate::{
-    hunk::{CfgHunkBuilder, DefaultInitTag, Hunk, HunkIniter},
+    hunk::Hunk,
     kernel::{
-        self,
-        cfg::{CfgBuilder, CfgMutexBuilder},
-        LockMutexError, MarkConsistentMutexError, MutexProtocol, TryLockMutexError,
+        mutex, prelude::*, traits, Cfg, LockMutexError, MarkConsistentMutexError, MutexProtocol,
+        TryLockMutexError,
     },
-    prelude::*,
+    sync::source::{DefaultSource, Source},
     utils::Init,
 };
 
-/// Configuration builder type for [`RecursiveMutex`].
-pub struct Builder<System, T, InitTag> {
-    mutex: CfgMutexBuilder<System>,
-    hunk: CfgHunkBuilder<System, MutexInner<T>, InitTag>,
+/// The definer (static builder) for [`StaticRecursiveMutex`][].
+#[doc = include_str!("../common.md")]
+pub struct Definer<System, Source> {
+    mutex: mutex::MutexDefiner<System>,
+    source: Source,
 }
 
 /// A recursive mutex, which can be locked by a task for multiple times
@@ -26,24 +26,87 @@ pub struct Builder<System, T, InitTag> {
 ///
 ///  - When trying to lock an abandoned mutex, the lock function will return
 ///    `Err(LockError::Abandoned(lock_guard))`. This state can be exited by
-///    calling [`RecursiveMutex::mark_consistent`].
+///    calling [`GenericRecursiveMutex::mark_consistent`].
 ///
-///  - Mutexes must be unlocked in a lock-reverse order. [`MutexGuard`]`::drop`
-///    might panic if this is violated.
+///  - Mutexes must be unlocked in a lock-reverse order.
+///    [`GenericMutexGuard`]`::drop` might panic if this is violated.
+///
+/// # Example
+///
+/// See [`StaticRecursiveMutex`].
 ///
 /// [`r3::kernel::Mutex`]: crate::kernel::Mutex
-pub struct RecursiveMutex<System, T> {
-    hunk: Hunk<System, MutexInner<T>>,
-    mutex: kernel::Mutex<System>,
+pub struct GenericRecursiveMutex<Cell, Mutex> {
+    cell: Cell,
+    mutex: Mutex,
 }
 
+/// A defined (statically created) [`GenericRecursiveMutex`].
+///
+/// # Example
+///
+#[doc = crate::tests::doc_test!(
+/// ```rust
+/// use core::cell::Cell;
+/// use r3::{kernel::StaticTask, sync::StaticRecursiveMutex};
+///
+/// struct Objects {
+///     mutex: StaticRecursiveMutex<System, Cell<i32>>,
+/// }
+///
+/// const fn configure_app<C>(cfg: &mut Cfg<C>) -> Objects
+/// where
+///     C: ~const traits::CfgBase<System = System> +
+///        ~const traits::CfgTask +
+///        ~const traits::CfgMutex,
+/// {
+///     StaticTask::define()
+///         .start(task1_body)
+///         .priority(2)
+///         .active(true)
+///         .finish(cfg);
+///
+///     let mutex = StaticRecursiveMutex::define()
+///         .init(|| Cell::new(1))
+///         .finish(cfg);
+///
+///     Objects { mutex }
+/// }
+///
+/// fn task1_body() {
+///     let guard = COTTAGE.mutex.lock().unwrap();
+///     assert_eq!(guard.get(), 1);
+///     guard.set(2);
+///
+///     {
+///         // Recursive lock is allowed
+///         let guard2 = COTTAGE.mutex.lock().unwrap();
+///         assert_eq!(guard2.get(), 2);
+///         guard2.set(3);
+///     }
+///
+///     assert_eq!(guard.get(), 3);
+/// #   exit(0);
+/// }
+/// ```
+)]
+pub type StaticRecursiveMutex<System, T> =
+    GenericRecursiveMutex<Hunk<System, MaybeUninit<MutexInner<T>>>, mutex::StaticMutex<System>>;
+
 // TODO: Test the panicking behavior on invalid unlock order
 // TODO: Test the abandonment behavior
+// TODO: Owned version
 
-unsafe impl<System: Kernel, T: 'static + Send> Send for RecursiveMutex<System, T> {}
-unsafe impl<System: Kernel, T: 'static + Send> Sync for RecursiveMutex<System, T> {}
+unsafe impl<Cell, Mutex, T: Send> Send for GenericRecursiveMutex<Cell, Mutex> where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>
+{
+}
+unsafe impl<Cell, Mutex, T: Send> Sync for GenericRecursiveMutex<Cell, Mutex> where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>
+{
+}
 
-#[doc(hidden)]
+/// The inner data structure of [`GenericRecursiveMutex`][].
 pub struct MutexInner<T> {
     /// A bit field containing *the nesting count* (`bits[1..BITS]`) and
     /// *an abandonment flag* (`bits[0]`, [`LEVEL_ABANDONED`]).
@@ -64,11 +127,34 @@ pub struct MutexInner<T> {
     data: T,
 }
 
+impl<T> MutexInner<T> {
+    /// Construct `MutexInner`.
+    #[inline]
+    pub const fn new(data: T) -> Self {
+        Self {
+            level: Cell::new(0),
+            data,
+        }
+    }
+}
+
 impl<T: Init> Init for MutexInner<T> {
-    const INIT: Self = Self {
-        level: Cell::new(0),
-        data: Init::INIT,
-    };
+    const INIT: Self = Self::new(T::INIT);
+}
+
+impl<T: ~const Default> const Default for MutexInner<T> {
+    #[inline]
+    fn default() -> Self {
+        Self::new(T::default())
+    }
+}
+
+/// Forwarded to [`Self::new`][].
+impl<T> const From<T> for MutexInner<T> {
+    #[inline]
+    fn from(x: T) -> Self {
+        Self::new(x)
+    }
 }
 
 /// The bit in [`MutexInner::level`] indicating that the nesting count is
@@ -82,25 +168,34 @@ const LEVEL_COUNT_SHIFT: u32 = 1;
 /// is dropped, the lock will be released.
 ///
 /// This structure is created by the [`lock`] and [`try_lock`] methods of
-/// [`RecursiveMutex`].
+/// [`GenericRecursiveMutex`].
 ///
-/// [`lock`]: RecursiveMutex::lock
-/// [`try_lock`]: RecursiveMutex::try_lock
-#[must_use = "if unused the RecursiveMutex will immediately unlock"]
-pub struct MutexGuard<'a, System: Kernel, T: 'static> {
-    mutex: &'a RecursiveMutex<System, T>,
+/// [`lock`]: GenericRecursiveMutex::lock
+/// [`try_lock`]: GenericRecursiveMutex::try_lock
+#[must_use = "if unused the GenericRecursiveMutex will immediately unlock"]
+pub struct GenericMutexGuard<'a, Cell, Mutex, T>
+where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>,
+    Mutex: mutex::MutexHandle,
+{
+    mutex: &'a GenericRecursiveMutex<Cell, Mutex>,
     _no_send_sync: PhantomData<*mut ()>,
 }
 
-unsafe impl<System: Kernel, T: 'static + Sync> Sync for MutexGuard<'_, System, T> {}
+unsafe impl<Cell, Mutex, T: Sync> Sync for GenericMutexGuard<'_, Cell, Mutex, T>
+where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>,
+    Mutex: mutex::MutexHandle,
+{
+}
 
-/// Type alias for the result of [`RecursiveMutex::lock`].
+/// Type alias for the result of [`GenericRecursiveMutex::lock`].
 pub type LockResult<Guard> = Result<Guard, LockError<Guard>>;
 
-/// Type alias for the result of [`RecursiveMutex::try_lock`].
+/// Type alias for the result of [`GenericRecursiveMutex::try_lock`].
 pub type TryLockResult<Guard> = Result<Guard, TryLockError<Guard>>;
 
-/// Error type of [`RecursiveMutex::lock`].
+/// Error type of [`GenericRecursiveMutex::lock`].
 #[repr(i8)]
 pub enum LockError<Guard> {
     /// CPU Lock is active, or the current context is not [waitable].
@@ -109,7 +204,7 @@ pub enum LockError<Guard> {
     BadContext = LockMutexError::BadContext as i8,
     /// The wait operation was interrupted by [`Task::interrupt`].
     ///
-    /// [`Task::interrupt`]: crate::kernel::Task::interrupt
+    /// [`Task::interrupt`]: crate::kernel::task::TaskMethods::interrupt
     Interrupted = LockMutexError::Interrupted as i8,
     /// The mutex was created with the protocol attribute having the value
     /// [`Ceiling`] and the current task's priority is higher than the
@@ -134,7 +229,7 @@ impl<Guard> fmt::Debug for LockError<Guard> {
     }
 }
 
-/// Error type of [`RecursiveMutex::try_lock`].
+/// Error type of [`GenericRecursiveMutex::try_lock`].
 #[repr(i8)]
 pub enum TryLockError<Guard> {
     /// CPU Lock is active, or the current context is not [a task context].
@@ -167,7 +262,7 @@ impl<Guard> fmt::Debug for TryLockError<Guard> {
     }
 }
 
-/// Error type of [`RecursiveMutex::mark_consistent`].
+/// Error type of [`GenericRecursiveMutex::mark_consistent`].
 #[derive(Debug)]
 #[repr(i8)]
 pub enum MarkConsistentError {
@@ -177,18 +272,24 @@ pub enum MarkConsistentError {
     Consistent = MarkConsistentMutexError::BadObjectState as i8,
 }
 
-impl<System: Kernel, T: 'static> RecursiveMutex<System, T> {
-    /// Construct a `Builder` to define a mutex in [a configuration
+impl<System, T: 'static> StaticRecursiveMutex<System, T>
+where
+    System: traits::KernelMutex + traits::KernelStatic,
+{
+    /// Construct a `Definer` to define a mutex in [a configuration
     /// function](crate#static-configuration).
-    pub const fn build() -> Builder<System, T, DefaultInitTag> {
-        Builder {
-            mutex: kernel::Mutex::build(),
-            hunk: Hunk::build(),
+    pub const fn define() -> Definer<System, DefaultSource<MutexInner<T>>> {
+        Definer {
+            mutex: mutex::MutexRef::define(),
+            source: DefaultSource::INIT, // [ref:default_source_is_default]
         }
     }
 }
 
-impl<System: Kernel, T: 'static, InitTag> Builder<System, T, InitTag> {
+impl<System, Source> Definer<System, Source>
+where
+    System: traits::KernelMutex + traits::KernelStatic,
+{
     /// Specify the mutex's protocol. Defaults to `None` when unspecified.
     pub const fn protocol(self, protocol: MutexProtocol) -> Self {
         Self {
@@ -198,39 +299,75 @@ impl<System: Kernel, T: 'static, InitTag> Builder<System, T, InitTag> {
     }
 }
 
-impl<System: Kernel, T: 'static, InitTag: HunkIniter<MutexInner<T>>> Builder<System, T, InitTag> {
+// Define methods to set `Definer::source`
+impl_source_setter!(
+    #[autowrap(MutexInner::new, MutexInner)]
+    impl Definer<System, #Source>
+);
+
+impl<System, Source> Definer<System, Source>
+where
+    System: traits::KernelMutex + traits::KernelStatic,
+{
     /// Complete the definition of a mutex, returning a reference to the mutex.
-    pub const fn finish(self, cfg: &mut CfgBuilder<System>) -> RecursiveMutex<System, T> {
-        RecursiveMutex {
-            hunk: self.hunk.finish(cfg),
+    // `CfgMutex` can't have `~const CfgBase` as a supertrait because of
+    // [ref:const_supertraits], hence we need to specify `~const CfgBase` here
+    pub const fn finish<C: ~const traits::CfgMutex<System = System> + ~const traits::CfgBase, T>(
+        self,
+        cfg: &mut Cfg<C>,
+    ) -> StaticRecursiveMutex<System, T>
+    where
+        Source: ~const self::Source<System, Target = MutexInner<T>>,
+    {
+        GenericRecursiveMutex {
+            // Safety: It's safe to unwrap `UnsafeCell` because there's already
+            // a `Cell` in `MutexInner` where it's needed. `T` is never mutably
+            // borrowed there.
+            cell: unsafe { self.source.into_unsafe_cell_hunk(cfg).transmute() },
             mutex: self.mutex.finish(cfg),
         }
     }
 }
 
-impl<System: Kernel, T: 'static> RecursiveMutex<System, T> {
+impl<Cell, Mutex, T> GenericRecursiveMutex<Cell, Mutex>
+where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>,
+    Mutex: mutex::MutexHandle,
+{
     /// Acquire the mutex, blocking the current thread until it is able to do
     /// so.
     ///
     /// # Panics
     ///
     /// This method will panic if the nesting count would overflow.
-    pub fn lock(&self) -> LockResult<MutexGuard<'_, System, T>> {
-        let level = &self.hunk.level;
+    pub fn lock(&self) -> LockResult<GenericMutexGuard<'_, Cell, Mutex, T>> {
+        let level;
 
         match self.mutex.lock() {
-            Ok(()) => {}
+            Ok(()) => {
+                // Safety: We are in a task, which means `self.cell` is
+                // initialized [ref:source_cell]
+                level = unsafe { &self.cell.assume_init_ref().level };
+            }
             Err(LockMutexError::WouldDeadlock) => {
+                // Safety: We are the owning task, which means `self.cell` is
+                // initialized [ref:source_cell]
+                level = unsafe { &self.cell.assume_init_ref().level };
                 level.update(|x| {
                     x.checked_add(1 << LEVEL_COUNT_SHIFT)
                         .expect("nesting count overflow")
                 });
             }
-            Err(LockMutexError::BadId) => unreachable!(),
+            Err(LockMutexError::NoAccess) => unreachable!(),
             Err(LockMutexError::BadContext) => return Err(LockError::BadContext),
             Err(LockMutexError::Interrupted) => return Err(LockError::Interrupted),
             Err(LockMutexError::BadParam) => return Err(LockError::BadParam),
             Err(LockMutexError::Abandoned) => {
+                // Safety: It being abandoned means there was a task that owned
+                // the lock, which means we are past the boot phase, which means
+                // `self.cell` is initialized [ref:source_cell]
+                level = unsafe { &self.cell.assume_init_ref().level };
+
                 // Make the nesting count consistent
                 level.set(LEVEL_ABANDONED);
                 self.mutex.mark_consistent().unwrap();
@@ -238,12 +375,12 @@ impl<System: Kernel, T: 'static> RecursiveMutex<System, T> {
         }
 
         if (level.get() & LEVEL_ABANDONED) != 0 {
-            Err(LockError::Abandoned(MutexGuard {
+            Err(LockError::Abandoned(GenericMutexGuard {
                 mutex: self,
                 _no_send_sync: PhantomData,
             }))
         } else {
-            Ok(MutexGuard {
+            Ok(GenericMutexGuard {
                 mutex: self,
                 _no_send_sync: PhantomData,
             })
@@ -255,22 +392,34 @@ impl<System: Kernel, T: 'static> RecursiveMutex<System, T> {
     /// # Panics
     ///
     /// This method will panic if the nesting count would overflow.
-    pub fn try_lock(&self) -> TryLockResult<MutexGuard<'_, System, T>> {
-        let level = &self.hunk.level;
+    pub fn try_lock(&self) -> TryLockResult<GenericMutexGuard<'_, Cell, Mutex, T>> {
+        let level;
 
         match self.mutex.try_lock() {
-            Ok(()) => {}
+            Ok(()) => {
+                // Safety: We are in a task, which means `self.cell` is
+                // initialized [ref:source_cell]
+                level = unsafe { &self.cell.assume_init_ref().level };
+            }
             Err(TryLockMutexError::WouldDeadlock) => {
+                // Safety: We are the owning task, which means `self.cell` is
+                // initialized [ref:source_cell]
+                level = unsafe { &self.cell.assume_init_ref().level };
                 level.update(|x| {
                     x.checked_add(1 << LEVEL_COUNT_SHIFT)
                         .expect("nesting count overflow")
                 });
             }
-            Err(TryLockMutexError::BadId) => unreachable!(),
+            Err(TryLockMutexError::NoAccess) => unreachable!(),
             Err(TryLockMutexError::BadContext) => return Err(TryLockError::BadContext),
             Err(TryLockMutexError::Timeout) => return Err(TryLockError::WouldBlock),
             Err(TryLockMutexError::BadParam) => return Err(TryLockError::BadParam),
             Err(TryLockMutexError::Abandoned) => {
+                // Safety: It being abandoned means there was a task that owned
+                // the lock, which means we are past the boot phase, which means
+                // `self.cell` is initialized [ref:source_cell]
+                level = unsafe { &self.cell.assume_init_ref().level };
+
                 // Make the nesting count consistent
                 level.set(LEVEL_ABANDONED);
                 self.mutex.mark_consistent().unwrap();
@@ -278,12 +427,12 @@ impl<System: Kernel, T: 'static> RecursiveMutex<System, T> {
         }
 
         if (level.get() & LEVEL_ABANDONED) != 0 {
-            Err(TryLockError::Abandoned(MutexGuard {
+            Err(TryLockError::Abandoned(GenericMutexGuard {
                 mutex: self,
                 _no_send_sync: PhantomData,
             }))
         } else {
-            Ok(MutexGuard {
+            Ok(GenericMutexGuard {
                 mutex: self,
                 _no_send_sync: PhantomData,
             })
@@ -292,18 +441,26 @@ impl<System: Kernel, T: 'static> RecursiveMutex<System, T> {
 
     /// Mark the state protected by the mutex as consistent.
     pub fn mark_consistent(&self) -> Result<(), MarkConsistentError> {
-        let level = &self.hunk.level;
-
         match self.mutex.mark_consistent() {
             Ok(()) => {
+                // Safety: It having been inconsistent means there was a task
+                // that owned a lock, which means we are past the boot phase,
+                // which means `self.cell` is initialized [ref:source_cell]
+                let level = unsafe { &self.cell.assume_init_ref().level };
+
                 // Make the nesting count consistent and mark the content as
                 // consistent at the same time
                 level.set(0);
                 Ok(())
             }
-            Err(MarkConsistentMutexError::BadId) => unreachable!(),
+            Err(MarkConsistentMutexError::NoAccess) => unreachable!(),
             Err(MarkConsistentMutexError::BadContext) => Err(MarkConsistentError::BadContext),
             Err(MarkConsistentMutexError::BadObjectState) => {
+                // Safety: CPU Lock is inactive, which means we are past the
+                // boot phase, which means `self.cell` is initialized
+                // [ref:source_cell]
+                let level = unsafe { &self.cell.assume_init_ref().level };
+
                 // The nesting count is consistent.
                 if (level.get() & LEVEL_ABANDONED) != 0 {
                     // Mark the content as consistent
@@ -320,15 +477,20 @@ impl<System: Kernel, T: 'static> RecursiveMutex<System, T> {
     /// Get a raw pointer to the contained data.
     #[inline]
     pub fn get_ptr(&self) -> *mut T {
-        core::ptr::addr_of!(self.hunk.data) as _
+        // Safety: Not really unsafe because we aren't borrowing anything
+        unsafe { core::ptr::addr_of!((*self.cell.as_ptr()).data) as *mut T }
     }
 }
 
-impl<System: Kernel, T: fmt::Debug + 'static> fmt::Debug for RecursiveMutex<System, T> {
+impl<Cell, Mutex, T: fmt::Debug> fmt::Debug for GenericRecursiveMutex<Cell, Mutex>
+where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>,
+    Mutex: mutex::MutexHandle,
+{
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self.try_lock() {
             Ok(guard) => f
-                .debug_struct("RecursiveMutex")
+                .debug_struct("GenericRecursiveMutex")
                 .field("data", &&*guard)
                 .finish(),
             Err(TryLockError::BadContext) => {
@@ -339,7 +501,7 @@ impl<System: Kernel, T: fmt::Debug + 'static> fmt::Debug for RecursiveMutex<Syst
                     }
                 }
 
-                f.debug_struct("RecursiveMutex")
+                f.debug_struct("GenericRecursiveMutex")
                     .field("data", &BadContextPlaceholder)
                     .finish()
             }
@@ -351,7 +513,7 @@ impl<System: Kernel, T: fmt::Debug + 'static> fmt::Debug for RecursiveMutex<Syst
                     }
                 }
 
-                f.debug_struct("RecursiveMutex")
+                f.debug_struct("GenericRecursiveMutex")
                     .field("data", &LockedPlaceholder)
                     .finish()
             }
@@ -363,7 +525,7 @@ impl<System: Kernel, T: fmt::Debug + 'static> fmt::Debug for RecursiveMutex<Syst
                     }
                 }
 
-                f.debug_struct("RecursiveMutex")
+                f.debug_struct("GenericRecursiveMutex")
                     .field("data", &AbandonedPlaceholder)
                     .finish()
             }
@@ -375,7 +537,7 @@ impl<System: Kernel, T: fmt::Debug + 'static> fmt::Debug for RecursiveMutex<Syst
                     }
                 }
 
-                f.debug_struct("RecursiveMutex")
+                f.debug_struct("GenericRecursiveMutex")
                     .field("data", &BadParamPlaceholder)
                     .finish()
             }
@@ -383,24 +545,38 @@ impl<System: Kernel, T: fmt::Debug + 'static> fmt::Debug for RecursiveMutex<Syst
     }
 }
 
-impl<System: Kernel, T: fmt::Debug + 'static> fmt::Debug for MutexGuard<'_, System, T> {
+impl<Cell, Mutex, T: fmt::Debug> fmt::Debug for GenericMutexGuard<'_, Cell, Mutex, T>
+where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>,
+    Mutex: mutex::MutexHandle,
+{
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         fmt::Debug::fmt(&**self, f)
     }
 }
 
-impl<System: Kernel, T: fmt::Display + 'static> fmt::Display for MutexGuard<'_, System, T> {
+impl<Cell, Mutex, T: fmt::Display> fmt::Display for GenericMutexGuard<'_, Cell, Mutex, T>
+where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>,
+    Mutex: mutex::MutexHandle,
+{
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         fmt::Display::fmt(&**self, f)
     }
 }
 
-/// The destructor of `MutexGuard` that releases the lock. It will panic if
+/// The destructor of `GenericMutexGuard` that releases the lock. It will panic if
 /// CPU Lock is active.
-impl<System: Kernel, T: 'static> Drop for MutexGuard<'_, System, T> {
+impl<Cell, Mutex, T> Drop for GenericMutexGuard<'_, Cell, Mutex, T>
+where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>,
+    Mutex: mutex::MutexHandle,
+{
     #[inline]
     fn drop(&mut self) {
-        let level = &self.mutex.hunk.level;
+        // Safety: We own the lock, which means we are past the boot phase,
+        // which means `self.mutex.cell` is initialized [ref:source_cell]
+        let level = unsafe { &self.mutex.cell.assume_init_ref().level };
         if level.get() == 0 || level.get() == LEVEL_ABANDONED {
             self.mutex.mutex.unlock().unwrap();
         } else {
@@ -409,10 +585,25 @@ impl<System: Kernel, T: 'static> Drop for MutexGuard<'_, System, T> {
     }
 }
 
-impl<System: Kernel, T: 'static> core::ops::Deref for MutexGuard<'_, System, T> {
+impl<Cell, Mutex, T> Deref for GenericMutexGuard<'_, Cell, Mutex, T>
+where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>,
+    Mutex: mutex::MutexHandle,
+{
     type Target = T;
     #[inline]
     fn deref(&self) -> &Self::Target {
-        &self.mutex.hunk.data
+        // Safety: We own the lock, which means we are past the boot phase,
+        // which means `self.mutex.cell` is initialized [ref:source_cell]
+        unsafe { &self.mutex.cell.assume_init_ref().data }
     }
 }
+
+// Safety: `GenericMutexGuard::deref` provides a stable address
+unsafe impl<Cell, Mutex, T> stable_deref_trait::StableDeref
+    for GenericMutexGuard<'_, Cell, Mutex, T>
+where
+    Cell: Deref<Target = MaybeUninit<MutexInner<T>>>,
+    Mutex: mutex::MutexHandle,
+{
+}
